/// @addtogroup capd
/// @{

/////////////////////////////////////////////////////////////////////////////
/// @file minmax.h
///
/// @author The CAPD Group
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2005 by the CAPD Group.
//
// This file is part of the CAPD library.  This library is free software;
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation;
// either version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this software; see the file "license.txt".  If not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.

/* min, max and abs definitions */

#ifndef _minmax_h_
#define _minmax_h_

#undef max
#undef min

class interval;

namespace capd{

   template<typename T>
   inline T min(const T& x, const T& y)
   {
      return (x<y ? x : y);
   }

   template<typename T>
   inline T max(const T& x, const T& y)
   {
      return (x<y ? y : x);
   }

   template<typename T>
   inline T abs(const T& x)
   {
      return x<0. ? -x : x;
   }

   // definitions in interval.h and interval.cpp
   interval min(const interval& x, const interval& y);
   interval max(const interval& x, const interval& y);
   interval abs(const interval& x);
} // namespace capd

#endif

/// @}
