/// @addtogroup homologicalAlgebra
/// @{

/////////////////////////////////////////////////////////////////////////////
/// @file ChainT.h
///
/// @author Marian Mrozek
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2006 by the CAPD Group.
//
// This file is part of the CAPD library.  This library is free software;
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation;
// either version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this software; see the file "license.txt".  If not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.

#if !defined(_CHAINT_H_)
#define _CHAINT_H_

template<class ChainContainer>
class ChainT : public ChainContainer{
  public:

  typedef typename ChainContainer::key_type GeneratorCodeType;
  typedef typename ChainContainer::mapped_type ScalarType;
  typedef typename ChainContainer::iterator iterator;
  typedef typename ChainContainer::const_iterator const_iterator;

  bool nonZeroAt(const GeneratorCodeType& gen) const{
    return count(gen);
  }

  ScalarType at(const GeneratorCodeType& A_g) const{
    const_iterator it=find(A_g);
    if(it==this->end()) return ScalarType(0);
    else return it->second;
  }

  ChainT& operator+=(const ChainT& A_chain2){
    for(const_iterator it=A_chain2.begin();it!=A_chain2.end();++it){
      const GeneratorCodeType& arg=it->first;
      const ScalarType& val=it->second;
      (*this)[arg]+=val;
      if((*this)[arg]==ScalarType(0)){
        erase(arg);
      }
    }
    return *this;
  }

};

template<class ChainContainer>
std::ostream& operator<<(std::ostream& out,const  ChainT<ChainContainer>& A_chain){
  typedef typename ChainT<ChainContainer>::const_iterator const_iterator;

  out << "{";
  for(const_iterator it=A_chain.begin();it!=A_chain.end();++it){
    if(it!=A_chain.begin()) out << ", ";
    out << it->first << "->" << it->second;
  }
  out << "}";
  return out;
}

template<typename ScalarType,class ChainContainer>
ChainT<ChainContainer> operator*(ScalarType A_s,const ChainT<ChainContainer>& A_chain){
  typedef typename ChainT<ChainContainer>::const_iterator const_iterator;

  ChainT<ChainContainer> result;
  if(A_s != ScalarType(0)){
    for(const_iterator it=A_chain.begin();it!=A_chain.end();++it){
      result.insert(std::make_pair(it->first,it->second*A_s));
    }
  }
  return result;
}

#endif //_CHAINT_H_
/// @}

