/// @addtogroup interval
/// @{

/////////////////////////////////////////////////////////////////////////////
/// @file intvswr.c
///
/// @author The CAPD Group
/////////////////////////////////////////////////////////////////////////////

// Copyright (C) 2000-2005 by the CAPD Group.
//
// This file is part of the CAPD library.  This library is free software;
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation;
// either version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this software; see the file "license.txt".  If not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.

//------------
//            a class 'Interval' with switched rounding
//------------


//Operator=
interval& interval::operator =(const interval& _inter)
{   if(this == &_inter)    // this a=a
       return *this;
    left  = _inter.left;
    right = _inter.right;
    return *this;
}

//Operator +=
interval& interval::operator +=(const interval& _inter)
{   round_down();
    left  += _inter.left;
    round_up();
    right += _inter.right;
    #ifdef  DEBUGGING
    if(left>right)
      interval_error(" Operator += ",left,right);
    #endif
    return *this;
}

//Operator  -=
interval& interval::operator -=(const interval& _inter)
{   round_down();
    left  -= _inter.right;
    round_up();
    right -= _inter.left;
    #ifdef  DEBUGGING
    if(left>right)
      interval_error(" Operator -= ",left,right);
    #endif
    return *this;
}

//Operator *=
interval& interval::operator *=(const interval& _inter)
{

    if((right==0.0) && (left==0.0)) return *this;
    if ((_inter.right==0.0) && (_inter.left==0.0)){
      left=0.0;
      right=0.0;
      return *this;
    }


    if(right<=0 &&  _inter.right<=0)  // (--)(--)
    {   round_down();
        bound_type temp=right*_inter.right;
        round_up();
        right=left*_inter.left;
        left=temp;
        #ifdef  DEBUGGING
        if(left>right)
          interval_error(" Operator *= (--)(--)",left,right);
        #endif
        return *this;
    }


   if( right<=0 && _inter.left<=0 && _inter.right>=0) // (--)(-+)
    {   round_down();
        bound_type temp=left*_inter.right;
        round_up();
        right=left*_inter.left;
        left=temp;
        #ifdef  DEBUGGING
        if(left>right)
          interval_error(" Operator *= (--)(-+)",left,right);
        #endif
        return *this;
    }


    if(right<=0 && _inter.left>=0 ) // (--)(++)
    {   round_down();
        left*=_inter.right;
        round_up();
        right*=_inter.left;
        #ifdef  DEBUGGING
        if(left>right)
          interval_error(" Operator *= (--)(++)",left,right);
        #endif
        return *this;
    }

   if(left<=0 && right>=0 &&  _inter.right<=0) // (-+)(--)
    {   round_down();
        bound_type temp=right*_inter.left;
        round_up();
        right=left*_inter.left;
        left=temp;
        #ifdef  DEBUGGING
        if(left>right)
          interval_error(" Operator *= (-+)(--)",left,right);
        #endif
        return *this;
    }


   if(left<=0 && right>=0 && _inter.left<=0 && _inter.right>=0)  // (-+)(-+)
    {  round_down();
       bound_type l1=left*_inter.right, l2=right*_inter.left;
       round_up();
       bound_type r1=left*_inter.left, r2=right*_inter.right;
       if(l1>l2) left=l2;
       else left=l1;
       if(r1<r2) right=r2;
       else right=r1;
       #ifdef  DEBUGGING
        if(left>right)
          interval_error(" Operator *= (-+)(-+)",left,right);
        #endif
       return *this;
    }



   if(left<=0 && right>=0 && _inter.left>=0 ) // (-+)(++)
    {   round_down();
        left*=_inter.right;
        round_up();
        right*=_inter.right;
        #ifdef  DEBUGGING
        if(left>right)
          interval_error(" Operator *= (-+)(++)",left,right);
        #endif
        return *this;
    }


    if(left>=0 &&  _inter.right<=0) // (++)(--)
    {   round_down();
        bound_type temp=right*_inter.left;
        round_up();
        right=left*_inter.right;
        left=temp;
        #ifdef  DEBUGGING
        if(left>right)
          interval_error(" Operator *= (++)(--)",left,right);
        #endif
        return *this;
    }

    if(left>=0  && _inter.left<=0 && _inter.right>=0) // (++)(-+)
    {   round_down();
        left=right*_inter.left;
        round_up();
        right*=_inter.right;
        #ifdef  DEBUGGING
        if(left>right)
          interval_error(" Operator *= (++)(-+)",left,right);
        #endif
        return *this;
    }


    if(left>=0 && _inter.right >=0){ // (++)(++)
      round_down();
      left*=_inter.left;
      round_up();
      right*=_inter.right;

      #ifdef  DEBUGGING
      if(left>right)
        interval_error(" Operator *= (++)(++)",left,right);
      #endif
      return *this;
    }


    interval_error("Error *** Fatal interval error ", left, right);
  return *this;
}

//Operator  /=
interval& interval::operator /=(const interval& _inter)
{
  if(_inter.left<=0 && _inter.right>=0)
    {
      interval_error("Error ***  Possible divide by zero.  /=", left, right);
    }


 if(right<=0 && _inter.right<0){  // (--)(--)
   round_down();
   bound_type ll=right/_inter.left;

   round_up();
   bound_type rr=left/_inter.right;
   left=ll;
   right=rr;
   #ifdef  DEBUGGING
    if(left>right)
      interval_error(" Operator /= (--)(--)",left,right);
    #endif
   return *this;
 }

 if(right<=0 && _inter.left>0)   // (--)(++)
    {   round_down();
        left/=_inter.left;
        round_up();
        right/=_inter.right;
        #ifdef  DEBUGGING
        if(left>right)
          interval_error(" Operator /= (--)(++)",left,right);
        #endif

        return *this;
    }


 if(left<=0 && right>=0 && _inter.right<0)  // (-+)(--)
    {   round_down();
        bound_type temp=right/_inter.right;
        round_up();
        right=left/_inter.right;
        left=temp;
        #ifdef  DEBUGGING
        if(left>right)
          interval_error(" Operator /= (-+)(--)",left,right);
        #endif
        return *this;
    }


 if(left<=0 && right>=0 && _inter.left>0)  // (-+)(++)
    {   round_down();
        left /=_inter.left;
        round_up();
        right/=_inter.left;
        #ifdef  DEBUGGING
        if(left>right)
          interval_error(" Operator /= (-+)(++)",left,right);
        #endif
        return *this;
    }

 if(left>=0 && _inter.right<0) // (++)(--)
    {   round_down();
        bound_type temp=right/_inter.right;
        round_up();
        right=left/_inter.left;
        left=temp;
        #ifdef  DEBUGGING
        if(left>right)
          interval_error(" Operator /= (++)(--)",left,right);
        #endif

        return *this;
    }

 if(left>=0 && _inter.left>0) // (++)(++)
    {   round_down();
        left/=_inter.right;
        round_up();
        right/=_inter.left;
        #ifdef  DEBUGGING
        if(left>right)
          interval_error(" Operator /= (++)(++)",left,right);
        #endif
        return *this;
    }

  interval_error("Error *** Fatal interval error in  /= ",left,right);

  return *this;
}

//Operator +
interval operator +(const interval& _inter1, const interval& _inter2)
{   bound_type ll,rr;
    round_down();
    ll = _inter1.left+_inter2.left;
    round_up();
    rr = _inter1.right+_inter2.right;
    return interval(ll,rr);
}
interval operator+(const interval& _inter, const bound_type _val)
{   bound_type ll,rr;
    round_down();
    ll = _inter.left+_val;
    round_up();
    rr = _inter.right+_val;
   return interval(ll,rr);
}
interval operator+(const bound_type _val,const interval& _inter)
{   bound_type ll,rr;
    round_down();
    ll = _inter.left+_val;
    round_up();
    rr = _inter.right+_val;
    return interval(ll, rr);
}//--------------------------------------------------------------------

//Operator  -
interval operator -(const interval& _inter1, const interval& _inter2)
{  bound_type ll,rr;
   round_down();
   ll = _inter1.left-_inter2.right;
   round_up();
   rr = _inter1.right-_inter2.left;
   return interval(ll,rr);
}
interval operator-(const interval& _inter, const bound_type _val)
{  bound_type ll,rr;
   round_down();
   ll = _inter.left-_val;
   round_up();
   rr = _inter.right-_val;
   return interval(ll,rr);
}
interval operator-(const bound_type _val,const interval& _inter)
{  bound_type ll,rr;
   round_down();
   ll = _val-_inter.right;
   round_up();
   rr = _val-_inter.left;
   return interval(ll,rr);
}//--------------------------------------------------------------------

//Operator *
interval  operator *(const interval& _inter1, const interval& _inter2)
{

    if((_inter1.left==0) && (_inter1.right==0)){
      return(interval(0.0,0.0));
    }

    if((_inter2.left==0) && (_inter2.right==0)){
      return(interval(0.0,0.0));
    }



    bound_type ll,rr,t;

    if ((_inter1.left >= 0) && (_inter2.left >=0)){ // (++)(++)
      round_down();
      ll=_inter1.left*_inter2.left;
      round_up();
      rr=_inter1.right*_inter2.right;
    } else
    if ((_inter1.right<=0)   && (_inter2.right <= 0)) { // (--)(--)
      round_down();
      ll=_inter1.right*_inter2.right;
      round_up();
      rr=_inter1.left*_inter2.left;
    }else
    if ((_inter1.right <=0) && (_inter2.left <= 0)){ // (--)(-+)
                     // _inter2.right > 0 - from previous condition
      round_down();
      ll=_inter1.left*_inter2.right;
      round_up();
      rr=_inter1.left*_inter2.left;
    } else
    if ((_inter1.right <=0) && (_inter2.left >=0)){ // (--)(++)
      round_down();
      ll=_inter1.left*_inter2.right;
      round_up();
      rr=_inter1.right*_inter2.left;
    } else    // _inter1.right > 0 - from previous conditions
    if ((_inter1.left <=0) && (_inter2.right <= 0)) { // (-+)(--)
      round_down();
      ll=_inter1.right*_inter2.left;
      round_up();
      rr=_inter1.left*_inter2.left;
    } else
    if ((_inter1.left <=0) && (_inter2.left <=0) ){ // (-+)(-+)
      round_down();
      ll=_inter1.left*_inter2.right;
      t=_inter2.left*_inter1.right;
      if (t<ll) ll=t;
      round_up();
      rr=_inter1.left*_inter2.left;
      t=_inter1.right*_inter2.right;
      if (t>rr) rr=t;
    } else
    if (_inter1.left <=0) { // (-+)(++)
      round_down();
      ll=_inter1.left*_inter2.right;
      round_up();
      rr=_inter1.right*_inter2.right;
    } else // _inter1.left > 0
    if (_inter2.right <=0) { // (++)(--)
      round_down();
      ll=_inter1.right*_inter2.left;
      round_up();
      rr=_inter1.left*_inter2.right;
    } else // _inter2.right > 0
    /*if(_inter2.left <=0)*/{ // (++)(-+)
      round_down();
      ll=_inter1.right*_inter2.left;
      round_up();
      rr=_inter1.right*_inter2.right;
    }

    #ifdef DEBUGGING
    if(ll>rr)
    {
      interval_error("Error *** Fatal interval error in operator*(interval, interval) .", _inter1.left, _inter1.right)
    }
    #endif
    return interval(ll,rr);
}

interval operator*(const interval& _inter, const bound_type _val)
{

   if((_inter.left==0) && (_inter.right==0)){
     return interval(0,0);
   }
   if(_val==0.0)      return interval(0,0);

   bound_type ll,rr;
   if(_val>=0)
   {  round_down();
      ll = _inter.left*_val;
      round_up();
      rr = _inter.right*_val;
   }
   else
   {  round_down();
      ll = _inter.right*_val;
      round_up();
      rr = _inter.left*_val;
   }
      return interval(ll, rr);
}

interval operator*(const bound_type _val,const interval& _inter)
{
   if((_inter.left==0) && (_inter.right==0)){
     return interval(0,0);
   }
   if(_val==0.0)      return interval(0,0);

   bound_type ll,rr;
   if(_val>=0)
   {  round_down();
      ll = _inter.left*_val;
      round_up();
      rr = _inter.right*_val;
   }
   else
   {  round_down();
      ll = _inter.right*_val;
      round_up();
      rr = _inter.left*_val;
   }
   return interval(ll,rr);
}//--------------------------------------------------------------------

//Operator  /
interval operator /(const interval& _inter1, const interval& _inter2)
{   bound_type ll,rr;
    if(_inter2.left<=0 && _inter2.right>=0)
    {
      interval_error("Interval Error *** Possible divide by zero.", _inter2.left,_inter2.right);
    }

    if(_inter1.right<=0 && _inter2.right<0) // (--)(--)
    {   round_down();
        ll=_inter1.right/_inter2.left;
        round_up();
        rr=_inter1.left/_inter2.right;
        #ifdef DEBUGGING
        if(ll>rr)
          interval_error(" Operator / (--)(--)",ll,rr);
        #endif
        return( interval(ll,rr));
    }

    if( _inter1.right<=0 && _inter2.left>0) // (--)(++)
    {   round_down();
        ll=_inter1.left/_inter2.left;
        round_up();
        rr=_inter1.right/_inter2.right;
        #ifdef DEBUGGING
        if(ll>rr)
          interval_error(" Operator / (--)(++)",ll,rr);
        #endif
        return( interval(ll,rr));
    }


    if(_inter1.left<=0 && _inter1.right>=0 && _inter2.right<0) // (-+)(--)
    {   round_down();
        ll=_inter1.right/_inter2.right;
        round_up();
        rr=_inter1.left/_inter2.right;
        #ifdef DEBUGGING
        if(ll>rr)
          interval_error(" Operator / (-+)(--)",ll,rr);
        #endif
        return( interval(ll,rr));
    }


    if(_inter1.left<=0 && _inter1.right>=0 && _inter2.left>0) // (-+)(++)
    {   round_down();
        ll=_inter1.left/_inter2.left;
        round_up();
        rr=_inter1.right/_inter2.left;
        #ifdef DEBUGGING
        if(ll>rr)
          interval_error(" Operator / (-+)(++)",ll,rr);
        #endif
        return( interval(ll,rr));
    }




    if(_inter1.left>=0 && _inter2.right<0)  // (++)(--)
    {   round_down();
        ll=_inter1.right/_inter2.right;
        round_up();
        rr=_inter1.left/_inter2.left;
        #ifdef DEBUGGING
        if(ll>rr)
          interval_error(" Operator / (++)(--)",ll,rr);
        #endif
        return( interval(ll,rr));
    }


    if(_inter1.left>=0 && _inter2.left>0)  // (++)(++)
    {   round_down();
        ll=_inter1.left/_inter2.right;
        round_up();
        rr=_inter1.right/_inter2.left;
        #ifdef DEBUGGING
        if(ll>rr)
          interval_error(" Operator / (++)(++)",ll,rr);
        #endif
        return( interval(ll,rr));
    }

    interval_error("Error *** Fatal interval error.",_inter2.left,_inter2.right);

  return(interval(0,0));
}//--------------------------------------------------------------------

//----------------------------------------- Operator %
interval operator %(const interval& _val1, const interval& _val2)
{

   if(_val2 <=0 )  // a divisor is either 0 or a negative number
   {
      interval_error("Error *** Operator %% - domain error",_val2.left,_val2.right);
   }

   interval temp = _val1;
   if( temp >= 0 )
     while( temp > _val2 )
       temp-=_val2;
   if( temp < 0 )
     while( temp < ( -1*_val2 ) )
       temp+=_val2;

   return temp;
}
interval operator %(const interval& _val1, const double _val2)
{
   if(_val2<=0)   // a divisor is either 0 or a negative number
   {
      interval_error("Error *** Operator %% - domain error",_val2,_val2);
   }

   interval temp=_val1;

   if( temp>= 0 )
      while( temp > _val2 )
        temp-=_val2;
   if( temp < 0 )
      while( temp < -_val2 )
        temp+=_val2;

   return temp;
}//--------------------------------------------------------------------


//-------- sqrt -----------------
interval sqrt(const interval & z)
{
  interval retval;

  if(z<0){
     interval_error("Error *** Interval Function sqrt(x) - domain error",z.left,z.right);
  }

  round_up();
  retval.right=sqrt(z.right);

  round_down();

  retval.left=sqrt(z.left);

 return retval;
}//-------------------------------


void interval::split(interval &r){
  double xl,xr;
  xl=left;
  xr=right;
  round_nearest();
  left=right=(xl+xr)/2;
  round_down();
  r.left=xl-left;
  round_up();
  r.right=xr-right;
  #ifdef DEBUGGING
  if(r.left>r.right)
      interval_error(" split",r.left,r.right);
  #endif
}

/// @}
