/// @addtogroup Matrix<int,0,0>Algorithms
/// @{
/////////////////////////////////////////////////////////////////////////////
/// @file intMatrixAlgorithms.cpp
///
/// @author Marian Mrozek
/////////////////////////////////////////////////////////////////////////////
// Copyright (C) 2000-2005 by the CAPD Group.
//
// This file is part of the CAPD library.  This library is free software;
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation;
// either version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this software; see the file "license.txt".  If not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.

#include "capd/minmax.h"
#include "matrixAlgorithms/intMatrixAlgorithms.hpp"
#include <cmath>
namespace capd{
  namespace matrixAlgorithms{

    /* Elementary row and column operations */
template void rowExchange<Matrix<int,0,0> >(Matrix<int,0,0>& A,int i,int j);
template void rowMultiply<Matrix<int,0,0> >(Matrix<int,0,0>& A,int i,Matrix<int,0,0>::ScalarType s);
template void rowAdd<Matrix<int,0,0> >(Matrix<int,0,0>& A,int i,int j,Matrix<int,0,0>::ScalarType s);
template void columnExchange<Matrix<int,0,0> >(Matrix<int,0,0>& A,int i,int j);
template void columnMultiply<Matrix<int,0,0> >(Matrix<int,0,0>& A,int j,Matrix<int,0,0>::ScalarType s);
template void columnAdd<Matrix<int,0,0> >(Matrix<int,0,0>& A,int i,int j,Matrix<int,0,0>::ScalarType s);
    /* Elementary row and column operations on Matrix<int,0,0> and matrices of bases */
template void rowExchange<Matrix<int,0,0>,Matrix<int,0,0> >(Matrix<int,0,0>& B,Matrix<int,0,0>& Q,Matrix<int,0,0>& Qinv,int i,int j);
template void rowMultiply<Matrix<int,0,0>,Matrix<int,0,0> >(Matrix<int,0,0>& B,Matrix<int,0,0>& Q,Matrix<int,0,0>& Qinv,int i,Matrix<int,0,0>::ScalarType q);
template void rowAdd<Matrix<int,0,0>,Matrix<int,0,0> >(Matrix<int,0,0>& B,Matrix<int,0,0>& Q,Matrix<int,0,0>& Qinv,int i,int j,Matrix<int,0,0>::ScalarType q);
template void columnExchange<Matrix<int,0,0>,Matrix<int,0,0> >(Matrix<int,0,0>& B,Matrix<int,0,0>& R,Matrix<int,0,0>& Rinv,int i,int j);
template void columnMultiply<Matrix<int,0,0>,Matrix<int,0,0> >(Matrix<int,0,0>& B,Matrix<int,0,0>& R,Matrix<int,0,0>& Rinv,int i,Matrix<int,0,0>::ScalarType q);
template void columnAdd<Matrix<int,0,0>,Matrix<int,0,0> >(Matrix<int,0,0>& B,Matrix<int,0,0>& R,Matrix<int,0,0>& Rinv,int i,int j,Matrix<int,0,0>::ScalarType q);
            // *** partial reduction *** //
template void partRowReduce<Matrix<int,0,0>,Matrix<int,0,0> >(Matrix<int,0,0>& B,Matrix<int,0,0>& Q,Matrix<int,0,0>& Qinv,int k,int l);
template void partColumnReduce<Matrix<int,0,0>,Matrix<int,0,0> >(Matrix<int,0,0>& B,Matrix<int,0,0>& R,Matrix<int,0,0>& Rinv,int k,int l);
      // *** Test for nonzero matrices *** /
template void smallestNonZero<Matrix<int,0,0> >(const Matrix<int,0,0>& A,Matrix<int,0,0>::ScalarType& s, int& iOpt, int& jOpt);
template bool nonZero<Matrix<int,0,0> >(const Matrix<int,0,0>& A);
          // *** row echelon form *** //
template void rowPrepare<Matrix<int,0,0>,Matrix<int,0,0> >(Matrix<int,0,0>& B,Matrix<int,0,0>& Q,Matrix<int,0,0>& Qinv,int k,int l);
template void rowReduce<Matrix<int,0,0>,Matrix<int,0,0> >(Matrix<int,0,0>& B,Matrix<int,0,0>& Q,Matrix<int,0,0>& Qinv,int k,int l);
template void rowEchelon<Matrix<int,0,0>,Matrix<int,0,0> >(Matrix<int,0,0>& B,Matrix<int,0,0>& Q,Matrix<int,0,0>& Qinv,int &k);
// *** column echelon form *** //
// *** under construction *** //
template void columnPrepare<Matrix<int,0,0>,Matrix<int,0,0> >(Matrix<int,0,0>& B,Matrix<int,0,0>& R,Matrix<int,0,0>& Rinv,int k,int l);
template void columnReduce<Matrix<int,0,0>,Matrix<int,0,0> >(Matrix<int,0,0>& B,Matrix<int,0,0>& R,Matrix<int,0,0>& Rinv,int k,int l);
template void columnEchelon<Matrix<int,0,0>,Matrix<int,0,0> >(Matrix<int,0,0>& B,Matrix<int,0,0>& R,Matrix<int,0,0>& Rinv,int &l);
// *** Smith diagonalization *** //
template void moveMinNonzero<Matrix<int,0,0>,Matrix<int,0,0>,Matrix<int,0,0> >(Matrix<int,0,0>& B,Matrix<int,0,0>& Q,Matrix<int,0,0>& Qinv,Matrix<int,0,0>& R,Matrix<int,0,0>& Rinv,int k);
template bool checkForDivisibility<Matrix<int,0,0> >(Matrix<int,0,0>& B,int k,int& i,int& j,Matrix<int,0,0>::ScalarType &q);
template void partSmithForm<Matrix<int,0,0>,Matrix<int,0,0>,Matrix<int,0,0> >(Matrix<int,0,0>& B,Matrix<int,0,0>& Q,Matrix<int,0,0>& Qinv,Matrix<int,0,0>& R,Matrix<int,0,0>& Rinv,int k);
template void smithForm<Matrix<int,0,0>,Matrix<int,0,0>,Matrix<int,0,0> >(Matrix<int,0,0>& B,Matrix<int,0,0>& Q,Matrix<int,0,0>& Qinv,Matrix<int,0,0>& R,Matrix<int,0,0>& Rinv,int &s,int &t);
  } // end of namespace matrixAlgorithms
} // end of namespace capd;
/// @}
