/// @addtogroup vectalg
/// @{
/////////////////////////////////////////////////////////////////////////////
/// @file ivector.cpp
///
/// @author The CAPD Group
/////////////////////////////////////////////////////////////////////////////
// Copyright (C) 2000-2005 by the CAPD Group.
//
// This file is part of the CAPD library.  This library is free software;
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation;
// either version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this software; see the file "license.txt".  If not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.

#include <iostream>

#include "interval/interval.h"
#include "vectalg/dimension.h"
#include "vectalg/vector.h"

template class Vector<interval,DIM>;

template Vector<interval,DIM> abs<interval,DIM> (const Vector<interval,DIM> &v);
template Vector<interval,DIM> operator- <interval,DIM> (const Vector<interval,DIM> &v);
template Vector<interval,DIM> operator+ <interval,DIM>(const Vector<interval,DIM> &v1,const Vector<interval,DIM> &v2);
template Vector<interval,DIM> operator- <interval,DIM>(const Vector<interval,DIM> &v1,const Vector<interval,DIM> &v2);
template interval operator* <interval,DIM> (const Vector<interval,DIM> &v1,const Vector<interval,DIM> &v2);

template Vector<interval,DIM> operator* <interval,interval,DIM>(const Vector<interval,DIM> &v,const interval &s);
template Vector<interval,DIM> operator* <interval,interval,DIM>(const interval &s,const Vector<interval,DIM> &v);
template Vector<interval,DIM> operator/ <interval,interval,DIM>(const Vector<interval,DIM> &v, const interval &s);

template Vector<interval,DIM> operator* <interval,double,DIM>(const Vector<interval,DIM> &v,const double &s);
template Vector<interval,DIM> operator* <interval,double,DIM>(const double &s,const Vector<interval,DIM> &v);
template Vector<interval,DIM> operator/ <interval,double,DIM>(const Vector<interval,DIM> &v, const double &s);

template Vector<interval,DIM> operator* <interval,long,DIM>(const Vector<interval,DIM> &v,const long &s);
template Vector<interval,DIM> operator* <interval,long,DIM>(const long &s,const Vector<interval,DIM> &v);
template Vector<interval,DIM> operator/ <interval,long,DIM>(const Vector<interval,DIM> &v, const long &s);

template Vector<interval,DIM> operator* <interval,int,DIM>(const Vector<interval,DIM> &v,const int &s);
template Vector<interval,DIM> operator* <interval,int,DIM>(const int &s,const Vector<interval,DIM> &v);
template Vector<interval,DIM> operator/ <interval,int,DIM>(const Vector<interval,DIM> &v, const int &s);

template Vector<interval,DIM> operator+ <interval,DIM> (const Vector<interval,DIM> &v, const interval &s);
template Vector<interval,DIM> operator- <interval,DIM>(const Vector<interval,DIM> &v,const interval &s);
template bool operator < <interval,DIM> (const Vector<interval,DIM> &v1,const Vector<interval,DIM> &v2);
template bool operator > <interval,DIM> (const Vector<interval,DIM> &v1,const Vector<interval,DIM> &v2);
template bool operator<= <interval,DIM> (const Vector<interval,DIM> &v1,const Vector<interval,DIM> &v2);
template bool operator>= <interval,DIM> (const Vector<interval,DIM> &v1,const Vector<interval,DIM> &v2);
template std::ostream &operator<< <interval,DIM> (std::ostream &out, const Vector<interval,DIM> &v);
template std::istream &operator>> <interval,DIM>(std::istream &inp, Vector<interval,DIM> &v);



typedef Vector<interval,DIM> IVector;

template IVector intervalHull<IVector>(const IVector &A, const IVector &B);
template void split<IVector>(IVector& v, IVector& rv);
template IVector midVector<IVector>(const IVector& v);
template IVector leftVector<IVector>(const IVector& v);
template IVector rightVector<IVector>(const IVector& v);
template interval size<IVector>(const IVector& v);
template IVector diam<IVector>(const IVector& v);
template IVector intervalBall<IVector>(const IVector &iv, const interval &r);
template interval solveAffineInclusion<IVector>(const IVector& a,const IVector& p,const IVector& c);
template interval solveAffineInclusion<IVector>(const IVector& a,const IVector& p,const IVector& c,int&);
template bool subset<IVector>(const IVector& v1, const IVector& v2);
template bool subsetInterior<IVector>(const IVector& v1, const IVector& v2);
template IVector intersection<IVector>(const IVector &v1, const IVector &v2);

/// @}
