/// @addtogroup vectalg
/// @{
/////////////////////////////////////////////////////////////////////////////
/// @file ivectorMD.cpp
///
/// @author The CAPD Group
/////////////////////////////////////////////////////////////////////////////
// Copyright (C) 2000-2005 by the CAPD Group.
//
// This file is part of the CAPD library.  This library is free software;
// you can redistribute it and/or modify it under the terms of the GNU
// General Public License as published by the Free Software Foundation;
// either version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this software; see the file "license.txt".  If not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
// MA 02111-1307, USA.

#include <iostream>
#include "interval/interval.h"
#include "vectalg/vector.h"

template class Vector<interval,0>;

template Vector<interval,0> abs<interval> (const Vector<interval,0> &v);
template Vector<interval,0> operator- <interval> (const Vector<interval,0> &v);
template Vector<interval,0> operator+ <interval>(const Vector<interval,0> &v1,const Vector<interval,0> &v2);
template Vector<interval,0> operator- <interval>(const Vector<interval,0> &v1,const Vector<interval,0> &v2);
template interval operator* <interval> (const Vector<interval,0> &v1,const Vector<interval,0> &v2);

template Vector<interval,0> operator* <interval,interval>(const Vector<interval,0> &v,const interval &s);
template Vector<interval,0> operator* <interval,interval>(const interval &s,const Vector<interval,0> &v);
template Vector<interval,0> operator/ <interval,interval>(const Vector<interval,0> &v, const interval &s);

template Vector<interval,0> operator* <interval,double>(const Vector<interval,0> &v,const double &s);
template Vector<interval,0> operator* <interval,double>(const double &s,const Vector<interval,0> &v);
template Vector<interval,0> operator/ <interval,double>(const Vector<interval,0> &v, const double &s);

template Vector<interval,0> operator* <interval,long>(const Vector<interval,0> &v,const long &s);
template Vector<interval,0> operator* <interval,long>(const long &s,const Vector<interval,0> &v);
template Vector<interval,0> operator/ <interval,long>(const Vector<interval,0> &v, const long &s);

template Vector<interval,0> operator* <interval,int>(const Vector<interval,0> &v,const int &s);
template Vector<interval,0> operator* <interval,int>(const int &s,const Vector<interval,0> &v);
template Vector<interval,0> operator/ <interval,int>(const Vector<interval,0> &v, const int &s);

template Vector<interval,0> operator+ <interval> (const Vector<interval,0> &v, const interval &s);
template Vector<interval,0> operator- <interval>(const Vector<interval,0> &v,const interval &s);
template bool operator < <interval> (const Vector<interval,0> &v1,const Vector<interval,0> &v2);
template bool operator > <interval> (const Vector<interval,0> &v1,const Vector<interval,0> &v2);
template bool operator<= <interval> (const Vector<interval,0> &v1,const Vector<interval,0> &v2);
template bool operator>= <interval> (const Vector<interval,0> &v1,const Vector<interval,0> &v2);
template std::ostream &operator<< <interval> (std::ostream &out, const Vector<interval,0> &v);
template std::istream &operator>> <interval>(std::istream &inp, Vector<interval,0> &v);


typedef Vector<interval,0> IVector;

template IVector intervalHull<IVector>(const IVector &A, const IVector &B);
template void split<IVector>(IVector& v, IVector& rv);
template IVector midVector<IVector>(const IVector& v);
template IVector leftVector<IVector>(const IVector& v);
template IVector rightVector<IVector>(const IVector& v);
template interval size<IVector>(const IVector& v);
template IVector diam<IVector>(const IVector& v);
template IVector intervalBall<IVector>(const IVector &iv, const interval &r);
template interval solveAffineInclusion<IVector>(const IVector& a,const IVector& p,const IVector& c);
template interval solveAffineInclusion<IVector>(const IVector& a,const IVector& p,const IVector& c,int&);
template bool subset<IVector>(const IVector& v1, const IVector& v2);
template bool subsetInterior<IVector>(const IVector& v1, const IVector& v2);
template IVector intersection<IVector>(const IVector &v1, const IVector &v2);

/// @}
